import java.util.HashMap;
import java.util.Vector;
import java.util.Iterator;

//Object representation of a graph
//@name: a string representing the name of the network
//@mapNodes: adjacency list of the graph, where the key of the map is the label of the node and the value is the object representation of the Node
public class Graph
{
	private String name;
	private HashMap<String,Node> mapNodes;
	public Graph(String name)
	{
		this.name=name;
		mapNodes=new HashMap<String,Node>();
	}
	//Check if graph is empty
	public boolean isEmpty()
	{
		return (numNodes()==0);
	}
	public int numNodes()
	{
		return mapNodes.size();
	}
	public int numEdges()
	{
		int numEdges=0;
		Iterator<String> it=mapNodes.keySet().iterator();
		while(it.hasNext())
		{
			String node=it.next();
			numEdges+=mapNodes.get(node).degree();
		}
		return numEdges/2;
	}
	//Check if there exists an edge between two nodes in the graph
	public boolean hasEdge(String idSource, String idDest)
	{
		if(mapNodes.get(idSource).getAdiacs().containsKey(idDest))
			return true;
		else
			return false;
	}
	public String getName()
	{
		return name;
	}
	public HashMap<String,Node> getMapNodes()
	{
		return mapNodes;
	}
	//Add a node with a given label (if not present)
	public void addNode(String id)
	{
		if(!mapNodes.containsKey(id))
		{
			Node x=new Node(id);
			mapNodes.put(id,x);
		}
	}
	//Add an edge without an associated weight
	public void addArc(String idSource, String idDest)
	{
		addArc(idSource,idDest,0.0);
	}
	//Add an edge with an associated weight
	//The edge is added iff the two nodes are present and the edges is not already present in the graph 
	public void addArc(String idSource, String idDest, double weight)
	{
		Node source=mapNodes.get(idSource);
		Node dest=mapNodes.get(idDest);
		if(mapNodes.containsKey(idSource) && mapNodes.containsKey(idDest))
		{
			mapNodes.get(idSource).getAdiacs().put(idDest,weight);
			mapNodes.get(idDest).getAdiacs().put(idSource,weight);
		}
	}
	//Build the subgraph induced from a set of nodes, i.e. the subgraph formed by only the edges of the graph connecting nodes in the set
	//@nodes: set of nodes in the graph
	public Graph buildInducedSubgraph(Vector<String> nodes)
	{
		int i=0, j=0;
		Graph subGr=new Graph(name);
		//Add nodes to the induced subgraph
		for(i=0;i<nodes.size();i++)
			subGr.addNode(nodes.get(i));
		//Add edges to the induced subgraph
		for(i=0;i<nodes.size();i++)
		{
			String source=nodes.get(i);
			HashMap<String,Double> adiac=mapNodes.get(source).getAdiacs();
			for(j=0;j<nodes.size();j++)
			{
				String dest=nodes.get(j);
				if(adiac.containsKey(dest))
					subGr.addArc(source,dest,adiac.get(dest));
			}
		}
		return subGr;
	}
	
	//Print the set of nodes and edges of the graph
	public String toString()
	{
		if(isEmpty())
			return "Empty graph";
		else 
		{
			String nodes="Nodes = {";
			String edges="Edges = {";
			Iterator<String> itSource=mapNodes.keySet().iterator();
			while(itSource.hasNext())
			{
				String idSource=itSource.next();
				nodes+=idSource+", ";
				HashMap<String,Double> adiac=mapNodes.get(idSource).getAdiacs();
				Iterator<String> itDest=adiac.keySet().iterator();
				while(itDest.hasNext())
				{
					String idDest=itDest.next();
					if(idSource.compareTo(idDest)<0)
						edges+="("+idSource+","+idDest+","+adiac.get(idDest)+"); ";
				}
			}
			if(nodes.length()!=9)
				nodes=nodes.substring(0,nodes.length()-2);
			nodes+="}\n";
			if(edges.length()!=9)
				edges=edges.substring(0,edges.length()-2);
			edges+="}\n";
			return nodes+edges;
		}
        }
}